@@ -27,17 +27,30 @@ use Digest::MD5;
 use Data::Dumper;
 use CGI::Simple::Cookie;
 use File::Spec;
-use parent qw(PSGI::Hector::Base);
+use Crypt::Simple;
 our $prefix = "HT";
 our $path = "/tmp";
+##############################################################################################################
+
+=head2 new()
+
+	my $session = PSGI::Hector::Session->new($hector)
+
+Reads or creates a new session for the visitor.
+
+The correct Set-Cookie header will be issued through the provided L<PSGI::Hector::Response> object.
+
+=cut
+
 ##############################################################################################################################
 sub new{	#constructor
 	my($class, $hector) = @_;
-	my $self = $class->SUPER::new();
-	$self->{'_hector'} = $hector;
-	$self->{'id'} = undef;
-	$self->{'error'} = "";
-	$self->{'vars'} = {};
+	my $self = {
+		'_hector' => $hector,
+		'id' => undef,
+		'vars' => {}
+	};
+	bless $self, $class;
 	$self->_readOrCreate();
 	return $self;
 }
@@ -45,28 +58,6 @@ sub new{	#constructor
 sub DESTROY{
 	__PACKAGE__->_expire();	#remove old sessions
 }
-#########################################################################################################################
-sub validate{	#runs the defined sub to see if this sesion is validate
-	my $self = shift;
-	if($self->getVar('remoteIp')){
-		my $env = $self->_getHector()->getEnv();
-		if($self->getVar('remoteIp') eq $env->{'REMOTE_ADDR'}){
-			if($self->getVar('scriptPath') && $self->getVar('scriptPath') eq $env->{'SCRIPT_NAME'}){
-				return 1;
-			}
-			else{
-				$self->_getHector()->getLog()->log("Session " . $self->getVar('scriptPath') . " <> " . $env->{'SCRIPT_NAME'}, 'debug');
-			}
-		}
-		else{
-			$self->_getHector()->getLog()->log("Session " . $self->getVar('remoteIp') . " <> " . $env->{'REMOTE_ADDR'}, 'debug');
-		}
-	}
-	else{
-		$self->_getHector()->getLog()->log("Session has no remote IP", 'debug');
-	}
-	return 0;
-}
 ################################################################################################################
 
 =head2 setVar()
@@ -83,109 +74,63 @@ sub setVar{	#stores a variable in the session
 	$self->_storeVar($name, $value);
 	return $self->_write();
 }
+################################################################################################################
+
+=head2 setSecretVar()
+
+	$s->setSecretVar('name', 'value');
+
+Takes two arguments, first the name of the variable then the value of the variable to store.
+
+The value is encrypted before being stored.
+
+=cut
+
+#############################################################
+sub setSecretVar{
+	my($self, $name, $value) = @_;
+	my $encrypted = encrypt($value);
+	$self->setVar($name, $encrypted);
+}
+##########################################################################################################################
+
+=head2 getVar()
+
+	my $value = $s->getVar('name')
+
+Retrieves the value which corresponds to the given variable name.
+
+=cut
+
 ##########################################################################################################################
 sub getVar{	#gets a stored variable from the session
 	my($self, $name) = @_;
 	if(defined($self->{'vars'}->{$name})){
 		return $self->{'vars'}->{$name};
 	}
-	else{return undef;}
+	return undef;
 }
-###########################################################################################################################
-sub setError{
-	my($self, $error) = @_;
-	$self->{'error'} = $error;	#save the error
-	return 1;
-}
-###########################################################################################################################
-sub getError{	#returns the last error
-	my $self = shift;
-	return $self->{'error'};
-}
-###########################################################################################################################
-sub getId{	#returns the session id
-	my $self = shift;
-	return $self->{'id'};
-}
-##############################################################################################################
+##########################################################################################################################
 
-=head2 create()
+=head2 getSecretVar()
 
-	$response = $wf->getResponse();
-	my $hashref = {
-		username => "bob"
-	};
-	$s->create($hashref, $response);
+	my $value = $s->getSecretVar('name')
 
-Creates a new session for the visitor.
+Retrieves the value which corresponds to the given variable name.
 
-This saves the contents of the given hash reference into the session.
-
-The correct Set-Cookie header will be issued through the provided L<PSGI::Hector::Response> object.
+The value is decrypted before being returned.
 
 =cut
 
-##############################################################################################################
-sub create{	#creates a server-side cookie for the session
-	my $self = shift;
-	my $result = 0;
-	$self->setError("");	#as we are starting a new session we clear any previous errors first
-	my $sessionId = time() * $$;	#time in seconds * process id
-	my $ctx = Digest::MD5->new;
-	$ctx->add($sessionId);
-	$sessionId = $self->_getPrefix() . $ctx->hexdigest;
-	$self->_setId($sessionId);	#remember the session id
-	my $env = $self->_getHector()->getEnv();
-	#set some initial values
-	$self->_storeVar('remoteIp', $env->{'REMOTE_ADDR'});
-	$self->_storeVar('scriptPath', $env->{'SCRIPT_NAME'});
-	if(!$self->getError()){	#all ok so far
-		my $cookie = $self->_setCookie(VALUE => $self->getId());
-		my $response = $self->_getHector()->getResponse();
-		$response->header("Set-Cookie" => $cookie);
-		$result = 1;
-	}
-	$result;
+###########################################################
+sub getSecretVar{
+	my($self, $name) = @_;
+	my $encrypted = $self->getVar($name);
+	decrypt($encrypted);
 }
-##############################################################################################################
-sub read{	#read an existing session
-	my $self = shift;
-	my $result = 0;
-	my $sessionId = $self->_getHector()->getRequest()->getCookie("SESSION");	#get the session id from the browser
-	if(defined($sessionId)){	#got a sessionid of some sort
-		my $prefix = $self->_getPrefix();
-		if($sessionId =~ m/^($prefix[a-f0-9]+)$/){	#filename valid
-			my $path = $self->_getPath();
-			my $sessionFile = File::Spec->catfile($path, $1);
-			if(open(SSIDE, "<", $sessionFile)){	#try to open the session file
-				my $contents = "";
-				while(<SSIDE>){	#read each line of the file
-					$contents .= $_;
-				}
-				close(SSIDE);
-				if($contents =~ m/^(\$VAR1 = \{.+\};)$/m){	#check session contents
-					my $validContents = $1; #untaint variable
-					my $VAR1;	#the session contents var
-					{
-						eval $validContents;
-					}
-					$self->{'vars'} = $VAR1;
-					$result = 1;
-					$self->_setId($sessionId);	#remember the session id
-				}
-				else{
-					$self->setError("Session contents invalid");
-				}
-			}
-			else{
-				$self->setError("Cant open session file: $sessionFile: $!");
-			}
-		}
-		else{
-			$self->setError("Session ID invalid: $sessionId");
-		}
-	}
-	return $result;
+###########################################################################################################################
+sub getId{	#returns the session id
+	shift->{'id'};
 }
 ###########################################################################################
 
@@ -199,40 +144,26 @@ Remove the current session from memory, disk and expire it in the browser.
 
 ###########################################################################################
 sub delete{	#remove a session
-	my($self, $response) = @_;
-	my $result = 0;
-	if($response){
-		my $sessionId = $self->getId();
-		my $prefix = $self->_getPrefix();
-		if($sessionId =~ m/^$prefix[a-f0-9]+$/){	#id valid
-			my $path = $self->_getPath();
-			my $sessionFile = File::Spec->catfile($path, $sessionId);
-			if(unlink($sessionFile)){
-				$self->_getHector()->getLog()->log("Deleted session: $sessionId", 'debug');
-				my $cookie = $self->_setCookie(EXPIRE => 'now');
-				$response->header("Set-Cookie" , => $cookie);
-				$self = undef;	#destroy this object
-				$result = 1;
-			}
-			else{
-				$self->setError("Could not delete session");
-			}
-		}
-		else{
-			$self->setError("Session ID invalid: $sessionId");
-		}
-	}
-	else{
-		$self->setError("No response given");
-	}
-	$result;
+	my($self) = shift;
+	my $sessionId = $self->getId();
+	die("Session ID invalid: $sessionId") unless $self->_isIdValid($sessionId);
+
+	my $path = $self->_getPath();
+	my $sessionFile = File::Spec->catfile($path, $sessionId);
+	die("Could not delete session") unless unlink($sessionFile);
+
+	$self->_getHector()->getLog()->log("Deleted session: $sessionId", 'debug');
+	my $cookie = $self->_setCookie(VALUE => $sessionId, EXPIRE => 'now');
+	my $response = $self->_getHector()->getResponse();
+	$response->header("Set-Cookie" => $cookie);
+	$self = undef;	#destroy this object
+	return 1;
 }
 ###############################################################################################################
 #private class method
 ###############################################################################################################
 sub _getHector{
-	my $self = shift;
-	return $self->{'_hector'};
+	shift->{'_hector'};
 }
 ###########################################################################################################################
 sub _setId{
@@ -256,13 +187,9 @@ sub _setCookie{
 		-httponly => 1,
 		-secure => $secure
 	);
-	if($cookie){
-		return $cookie->as_string();
-	}
-	else{
-		$self->setError("Can't create cookie");
-	}
-	return undef;
+	die("Can't create cookie") unless $cookie;
+	
+	return $cookie->as_string();
 }
 ##############################################################################################################
 sub _expire{	#remove old session files
@@ -271,10 +198,9 @@ sub _expire{	#remove old session files
 	if(opendir(COOKIES, $path)){
 		my @sessions = readdir(COOKIES);
 		my $expire = (time - 86400);
-		foreach(@sessions){	#check each of the cookies
-			my $prefix = $self->_getPrefix();
-			if($_ =~ m/^($prefix[a-f0-9]+)$/){	#found a cookie file
-				my $sessionFile = File::Spec->catfile($path, $1);
+		foreach my $id (@sessions){	#check each of the cookies
+			if($self->_isIdValid($id)){	#found a cookie file
+				my $sessionFile = File::Spec->catfile($path, $id);
 				my @stat = stat($sessionFile);
 				if(defined($stat[9]) && $stat[9] < $expire){	#cookie is more than a day old, so remove it
 					unlink $sessionFile;
@@ -286,28 +212,88 @@ sub _expire{	#remove old session files
 }
 ############################################################################################################
 #private methods
+#########################################################################################################################
+sub _validate{	#runs the defined sub to see if this sesion is validate
+	my $self = shift;
+	my $sessionIp = $self->getVar('remoteIp');
+	unless($sessionIp){
+		$self->_getHector()->getLog()->log("Session has no remote IP", 'debug');
+		return 0;
+	}	
+	my $env = $self->_getHector()->getEnv();
+	if($sessionIp ne $env->{'REMOTE_ADDR'}){
+		$self->_getHector()->getLog()->log("Session " . $sessionIp . " <> " . $env->{'REMOTE_ADDR'}, 'debug');
+		return 0;
+	}
+	return 1;
+}
+##############################################################################################################
+sub _create{	#creates a server-side cookie for the session
+	my $self = shift;
+	my $sessionId = time() * $$;	#time in seconds * process id
+	my $ctx = Digest::MD5->new;
+	$ctx->add($sessionId);
+	$sessionId = $self->_getPrefix() . $ctx->hexdigest;
+	$self->_setId($sessionId);	#remember the session id
+	my $env = $self->_getHector()->getEnv();
+	#set some initial values
+	$self->setVar('remoteIp', $env->{'REMOTE_ADDR'});
+	$self->setVar('scriptPath', $env->{'SCRIPT_NAME'});
+	my $cookie = $self->_setCookie(VALUE => $self->getId());
+	my $response = $self->_getHector()->getResponse();
+	$response->header("Set-Cookie" => $cookie);
+	return 1;
+}
+##############################################################################################################
+sub _read{	#read an existing session
+	my $self = shift;
+	my $result = 0;
+	my $sessionId = $self->_getHector()->getRequest()->getCookie("SESSION");	#get the session id from the browser
+	return 0 unless defined($sessionId);	#got a sessionid of some sort
+
+	return 0 unless $self->_isIdValid($sessionId);	#filename valid
+
+	my $path = $self->_getPath();
+	my $sessionFile = File::Spec->catfile($path, $sessionId);
+	return 0 unless open(SSIDE, "<", $sessionFile);	#try to open the session file
+
+	my $contents = "";
+	while(<SSIDE>){	#read each line of the file
+		$contents .= $_;
+	}
+	close(SSIDE);
+	unless($contents =~ m/^(\$VAR1 = \{.+\};)$/m){	#check session contents
+		$self->_getHector()->getLog()->log('Session contents invalid', 'warn');
+		return 0;
+	}
+	
+	my $validContents = $1; #untaint variable
+	my $VAR1;	#the session contents var
+	{
+		eval $validContents;
+	}
+	$self->{'vars'} = $VAR1;
+	$self->_setId($sessionId);	#remember the session id
+	return 1;
+}
 ###########################################################################################
 sub _write{	#writes a server-side cookie for the session
 	my $self = shift;
-	my $prefix = $self->_getPrefix();
-	if($self->getId() =~ m/^($prefix[a-f0-9]+)$/){	#filename valid
-		my $path = $self->_getPath();
-		my $sessionFile = File::Spec->catfile($path, $1);
-		if(open(SSIDE, ">", $sessionFile)){
-			$Data::Dumper::Freezer = 'freeze';
-			$Data::Dumper::Toaster = 'toast';
-			$Data::Dumper::Indent = 0;	#turn off formatting
-			my $dump = Dumper $self->{'vars'};
-			if($dump){	#if we have any data
-				print SSIDE $dump;
-			}
-			close(SSIDE);
-		}
-		else{$self->setError("Cant write session: $!");}
+	my $sessionId = $self->getId();
+	die('Session ID invalid') unless $self->_isIdValid($sessionId);	#filename valid
+
+	my $sessionFile = File::Spec->catfile($self->_getPath(), $sessionId);
+	die("Cant write session: $!") unless open(SSIDE, ">", $sessionFile);
+	
+	$Data::Dumper::Freezer = 'freeze';
+	$Data::Dumper::Toaster = 'toast';
+	$Data::Dumper::Indent = 0;	#turn off formatting
+	my $dump = Dumper $self->{'vars'};
+	if($dump){	#if we have any data
+		print SSIDE $dump;
 	}
-	else{$self->setError('Session ID invalid');}
-	if($self->getError()){return 0;}
-	else{return 1;}
+	close(SSIDE);
+	return 1;
 }
 ##########################################################################################################################
 sub _storeVar{	#stores a variable in the session
@@ -335,13 +321,19 @@ sub _getPath{	#this should be a config option
 #####################################################################################################################
 sub _readOrCreate{
 	my $self = shift;
-	if($self->read() && $self->validate()){
+	if($self->_read() && $self->_validate()){
 		$self->_getHector()->getLog()->log("Existing session: " . $self->getId(), 'debug');
 	}
-	elsif($self->create()){	#start a new session
+	elsif($self->_create()){	#start a new session
 		$self->_getHector()->getLog()->log("Created new session: " . $self->getId(), 'debug');
 	}
 }
+#######################################################################################################
+sub _isIdValid{
+	my($self, $id) = @_;
+	my $prefix = $self->_getPrefix();
+	$id =~ m/^($prefix[a-f0-9]+)$/;
+}
 #####################################################################################################################
 
 =pod
@@ -358,7 +350,7 @@ Development questions, bug reports, and patches are welcome to the above address
 
 =head1 Copyright
 
-Copyright (c) 2017 MacGyveR. All rights reserved.
+Copyright (c) 2020 MacGyveR. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
